@@ -28,7 +28,7 @@ class ListFloatingIPPool(lister.Lister):
     log = logging.getLogger(__name__ + '.ListFloatingIPPool')
 
     def take_action(self, parsed_args):
-        self.log.debug('take_action(%s)' % parsed_args)
+        self.log.debug('take_action(%s)', parsed_args)
         compute_client = self.app.client_manager.compute
 
         columns = ('Name',)
