@@ -24,6 +24,7 @@ import traceback
 
 from cliff import app
 from cliff import command
+from cliff import complete
 from cliff import help
 
 import openstackclient
@@ -64,8 +65,10 @@ class OpenStackShell(app.App):
     def __init__(self):
         # Patch command.Command to add a default auth_required = True
         command.Command.auth_required = True
+        command.Command.best_effort = False
         # But not help
         help.HelpCommand.auth_required = False
+        complete.CompleteCommand.best_effort = True
 
         super(OpenStackShell, self).__init__(
             description=__doc__.strip(),
@@ -117,9 +120,53 @@ class OpenStackShell(app.App):
                     action='store_true',
                     dest='deferred_help',
                     default=False,
-                    help="show this help message and exit",
+                    help="Show this help message and exit",
                 )
 
+    def configure_logging(self):
+        """Configure logging for the app
+
+        Cliff sets some defaults we don't want so re-work it a bit
+        """
+
+        if self.options.debug:
+            # --debug forces verbose_level 3
+            # Set this here so cliff.app.configure_logging() can work
+            self.options.verbose_level = 3
+
+        super(OpenStackShell, self).configure_logging()
+        root_logger = logging.getLogger('')
+
+        # Requests logs some stuff at INFO that we don't want
+        # unless we have DEBUG
+        requests_log = logging.getLogger("requests")
+        requests_log.setLevel(logging.ERROR)
+
+        # Other modules we don't want DEBUG output for so
+        # don't reset them below
+        iso8601_log = logging.getLogger("iso8601")
+        iso8601_log.setLevel(logging.ERROR)
+
+        # Set logging to the requested level
+        self.dump_stack_trace = False
+        if self.options.verbose_level == 0:
+            # --quiet
+            root_logger.setLevel(logging.ERROR)
+        elif self.options.verbose_level == 1:
+            # This is the default case, no --debug, --verbose or --quiet
+            root_logger.setLevel(logging.WARNING)
+        elif self.options.verbose_level == 2:
+            # One --verbose
+            root_logger.setLevel(logging.INFO)
+        elif self.options.verbose_level >= 3:
+            # Two or more --verbose
+            root_logger.setLevel(logging.DEBUG)
+            requests_log.setLevel(logging.DEBUG)
+
+        if self.options.debug:
+            # --debug forces traceback
+            self.dump_stack_trace = True
+
     def run(self, argv):
         try:
             return super(OpenStackShell, self).run(argv)
@@ -143,11 +190,26 @@ class OpenStackShell(app.App):
             metavar='<auth-url>',
             default=env('OS_AUTH_URL'),
             help='Authentication URL (Env: OS_AUTH_URL)')
+        parser.add_argument(
+            '--os-domain-name',
+            metavar='<auth-domain-name>',
+            default=env('OS_DOMAIN_NAME'),
+            help='Domain name of the requested domain-level '
+                 'authorization scope (Env: OS_DOMAIN_NAME)',
+        )
+        parser.add_argument(
+            '--os-domain-id',
+            metavar='<auth-domain-id>',
+            default=env('OS_DOMAIN_ID'),
+            help='Domain ID of the requested domain-level '
+                 'authorization scope (Env: OS_DOMAIN_ID)',
+        )
         parser.add_argument(
             '--os-project-name',
             metavar='<auth-project-name>',
             default=env('OS_PROJECT_NAME', default=env('OS_TENANT_NAME')),
-            help='Authentication project name (Env: OS_PROJECT_NAME)',
+            help='Project name of the requested project-level '
+                 'authorization scope (Env: OS_PROJECT_NAME)',
         )
         parser.add_argument(
             '--os-tenant-name',
@@ -159,7 +221,8 @@ class OpenStackShell(app.App):
             '--os-project-id',
             metavar='<auth-project-id>',
             default=env('OS_PROJECT_ID', default=env('OS_TENANT_ID')),
-            help='Authentication project ID (Env: OS_PROJECT_ID)',
+            help='Project ID of the requested project-level '
+                 'authorization scope (Env: OS_PROJECT_ID)',
         )
         parser.add_argument(
             '--os-tenant-id',
@@ -177,6 +240,30 @@ class OpenStackShell(app.App):
             metavar='<auth-password>',
             default=utils.env('OS_PASSWORD'),
             help='Authentication password (Env: OS_PASSWORD)')
+        parser.add_argument(
+            '--os-user-domain-name',
+            metavar='<auth-user-domain-name>',
+            default=utils.env('OS_USER_DOMAIN_NAME'),
+            help='Domain name of the user (Env: OS_USER_DOMAIN_NAME)')
+        parser.add_argument(
+            '--os-user-domain-id',
+            metavar='<auth-user-domain-id>',
+            default=utils.env('OS_USER_DOMAIN_ID'),
+            help='Domain ID of the user (Env: OS_USER_DOMAIN_ID)')
+        parser.add_argument(
+            '--os-project-domain-name',
+            metavar='<auth-project-domain-name>',
+            default=utils.env('OS_PROJECT_DOMAIN_NAME'),
+            help='Domain name of the project which is the requested '
+                 'project-level authorization scope '
+                 '(Env: OS_PROJECT_DOMAIN_NAME)')
+        parser.add_argument(
+            '--os-project-domain-id',
+            metavar='<auth-project-domain-id>',
+            default=utils.env('OS_PROJECT_DOMAIN_ID'),
+            help='Domain ID of the project which is the requested '
+                 'project-level authorization scope '
+                 '(Env: OS_PROJECT_DOMAIN_ID)')
         parser.add_argument(
             '--os-region-name',
             metavar='<auth-region-name>',
@@ -284,11 +371,16 @@ class OpenStackShell(app.App):
                         " either --os-password, or env[OS_PASSWORD], "
                         " or prompted response")
 
-            if not (self.options.os_project_id
-                    or self.options.os_project_name):
+            if not ((self.options.os_project_id
+                    or self.options.os_project_name) or
+                    (self.options.os_domain_id
+                    or self.options.os_domain_name)):
                 raise exc.CommandError(
-                    "You must provide a project id via"
-                    " either --os-project-id or via env[OS_PROJECT_ID]")
+                    "You must provide authentication scope as a project "
+                    "or a domain via --os-project-id or env[OS_PROJECT_ID], "
+                    "--os-project-name or env[OS_PROJECT_NAME], "
+                    "--os-domain-id or env[OS_DOMAIN_ID], or"
+                    "--os-domain-name or env[OS_DOMAIN_NAME].")
 
             if not self.options.os_auth_url:
                 raise exc.CommandError(
@@ -299,8 +391,14 @@ class OpenStackShell(app.App):
             token=self.options.os_token,
             url=self.options.os_url,
             auth_url=self.options.os_auth_url,
+            domain_id=self.options.os_domain_id,
+            domain_name=self.options.os_domain_name,
             project_name=self.options.os_project_name,
             project_id=self.options.os_project_id,
+            user_domain_id=self.options.os_user_domain_id,
+            user_domain_name=self.options.os_user_domain_name,
+            project_domain_id=self.options.os_project_domain_id,
+            project_domain_name=self.options.os_project_domain_name,
             username=self.options.os_username,
             password=self.options.os_password,
             region_name=self.options.os_region_name,
@@ -347,15 +445,6 @@ class OpenStackShell(app.App):
 
         super(OpenStackShell, self).initialize_app(argv)
 
-        # Set requests logging to a useful level
-        requests_log = logging.getLogger("requests")
-        if self.options.debug:
-            requests_log.setLevel(logging.DEBUG)
-            self.dump_stack_trace = True
-        else:
-            requests_log.setLevel(logging.WARNING)
-            self.dump_stack_trace = False
-
         # Save default domain
         self.default_domain = self.options.os_default_domain
 
@@ -368,13 +457,14 @@ class OpenStackShell(app.App):
             ver = getattr(self.options, mod.API_VERSION_OPTION, None)
             if ver:
                 self.api_version[mod.API_NAME] = ver
-                self.log.debug('%s API version %s' % (mod.API_NAME, ver))
+                self.log.debug('%(name)s API version %(version)s',
+                               {'name': mod.API_NAME, 'version': ver})
 
         # Add the API version-specific commands
         for api in self.api_version.keys():
             version = '.v' + self.api_version[api].replace('.', '_')
             cmd_group = 'openstack.' + api.replace('-', '_') + version
-            self.log.debug('command group %s' % cmd_group)
+            self.log.debug('command group %s', cmd_group)
             self.command_manager.add_command_group(cmd_group)
 
         # Commands that span multiple APIs
@@ -414,7 +504,15 @@ class OpenStackShell(app.App):
         """Set up auth and API versions"""
         self.log.debug('prepare_to_run_command %s', cmd.__class__.__name__)
 
-        if cmd.auth_required:
+        if not cmd.auth_required:
+            return
+        if cmd.best_effort:
+            try:
+                self.authenticate_user()
+                self.restapi.set_auth(self.client_manager.identity.auth_token)
+            except Exception:
+                pass
+        else:
             self.authenticate_user()
             self.restapi.set_auth(self.client_manager.identity.auth_token)
         return
