@@ -14,6 +14,7 @@
 #
 
 import copy
+import mock
 
 from openstackclient.identity.v2_0 import user
 from openstackclient.tests import fakes
@@ -99,6 +100,7 @@ class TestUserCreate(TestUser):
         ]
         verifylist = [
             ('name', identity_fakes.user_name),
+            ('password_prompt', False),
             ('password', 'secret')
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -130,6 +132,47 @@ class TestUserCreate(TestUser):
         )
         self.assertEqual(data, datalist)
 
+    def test_user_create_password_prompt(self):
+        arglist = [
+            '--password-prompt',
+            identity_fakes.user_name,
+        ]
+        verifylist = [
+            ('name', identity_fakes.user_name),
+            ('password_prompt', True)
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # DisplayCommandBase.take_action() returns two tuples
+        mocker = mock.Mock()
+        mocker.return_value = 'abc123'
+        with mock.patch("openstackclient.common.utils.get_password", mocker):
+            columns, data = self.cmd.take_action(parsed_args)
+
+        # Set expected values
+        kwargs = {
+            'enabled': True,
+            'tenant_id': None,
+        }
+        # UserManager.create(name, password, email, tenant_id=, enabled=)
+        self.users_mock.create.assert_called_with(
+            identity_fakes.user_name,
+            'abc123',
+            None,
+            **kwargs
+        )
+
+        collist = ('email', 'enabled', 'id', 'name', 'project_id')
+        self.assertEqual(columns, collist)
+        datalist = (
+            identity_fakes.user_email,
+            True,
+            identity_fakes.user_id,
+            identity_fakes.user_name,
+            identity_fakes.project_id,
+        )
+        self.assertEqual(data, datalist)
+
     def test_user_create_email(self):
         arglist = [
             '--email', 'barney@example.com',
@@ -338,6 +381,11 @@ class TestUserList(TestUser):
     def setUp(self):
         super(TestUserList, self).setUp()
 
+        self.projects_mock.get.return_value = fakes.FakeResource(
+            None,
+            copy.deepcopy(identity_fakes.PROJECT_2),
+            loaded=True,
+        )
         self.projects_mock.list.return_value = [
             fakes.FakeResource(
                 None,
@@ -365,7 +413,7 @@ class TestUserList(TestUser):
         # DisplayCommandBase.take_action() returns two tuples
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.users_mock.list.assert_called_with()
+        self.users_mock.list.assert_called_with(tenant_id=None)
 
         collist = ('ID', 'Name')
         self.assertEqual(columns, collist)
@@ -383,11 +431,12 @@ class TestUserList(TestUser):
             ('project', identity_fakes.project_id),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+        project_id = identity_fakes.PROJECT_2['id']
 
         # DisplayCommandBase.take_action() returns two tuples
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.users_mock.list.assert_called_with()
+        self.users_mock.list.assert_called_with(tenant_id=project_id)
 
         collist = ('ID', 'Name')
         self.assertEqual(columns, collist)
@@ -409,7 +458,7 @@ class TestUserList(TestUser):
         # DisplayCommandBase.take_action() returns two tuples
         columns, data = self.cmd.take_action(parsed_args)
 
-        self.users_mock.list.assert_called_with()
+        self.users_mock.list.assert_called_with(tenant_id=None)
 
         collist = ('ID', 'Name', 'Project', 'Email', 'Enabled')
         self.assertEqual(columns, collist)
@@ -498,6 +547,7 @@ class TestUserSet(TestUser):
         verifylist = [
             ('name', None),
             ('password', 'secret'),
+            ('password_prompt', False),
             ('email', None),
             ('project', None),
             ('enable', False),
@@ -515,6 +565,35 @@ class TestUserSet(TestUser):
             'secret',
         )
 
+    def test_user_set_password_prompt(self):
+        arglist = [
+            '--password-prompt',
+            identity_fakes.user_name,
+        ]
+        verifylist = [
+            ('name', None),
+            ('password', None),
+            ('password_prompt', True),
+            ('email', None),
+            ('project', None),
+            ('enable', False),
+            ('disable', False),
+            ('user', identity_fakes.user_name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # DisplayCommandBase.take_action() returns two tuples
+        mocker = mock.Mock()
+        mocker.return_value = 'abc123'
+        with mock.patch("openstackclient.common.utils.get_password", mocker):
+            self.cmd.take_action(parsed_args)
+
+        # UserManager.update_password(user, password)
+        self.users_mock.update_password.assert_called_with(
+            identity_fakes.user_id,
+            'abc123',
+        )
+
     def test_user_set_email(self):
         arglist = [
             '--email', 'barney@example.com',
